]> git.armaanb.net Git - slock.git/blobdiff - slock.c
applied two patches, BSD_AUTH patch and Gottox' DPMS support patch
[slock.git] / slock.c
diff --git a/slock.c b/slock.c
index daa3cf76b2e759f732de923ffa71a77da054b7af..2dc96483426b360fe8c051e7776c13cf0530f35f 100644 (file)
--- a/slock.c
+++ b/slock.c
@@ -1,4 +1,4 @@
/* © 2006-2007 Anselm R. Garbe <garbeam at gmail dot com>
  * See LICENSE file for license details. */
 #define _XOPEN_SOURCE 500
 #if HAVE_SHADOW_H
@@ -7,6 +7,7 @@
 
 #include <ctype.h>
 #include <pwd.h>
+#include <stdarg.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <X11/keysym.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
+#include <X11/extensions/dpms.h>
 
+#if HAVE_BSD_AUTH
+#include <login_cap.h>
+#include <bsd_auth.h>
+#endif
+
+void
+eprint(const char *errstr, ...) {
+       va_list ap;
+
+       va_start(ap, errstr);
+       vfprintf(stderr, errstr, ap);
+       va_end(ap);
+       exit(EXIT_FAILURE);
+}
+
+#ifndef HAVE_BSD_AUTH
 const char *
 get_password() { /* only run as root */
        const char *rval;
        struct passwd *pw;
 
-       if(geteuid() != 0) {
-               fputs("slock: cannot retrieve password entry (make sure to suid slock)\n", stderr);
-               exit(EXIT_FAILURE);
-       }
+       if(geteuid() != 0)
+               eprint("slock: cannot retrieve password entry (make sure to suid slock)\n");
        pw = getpwuid(getuid());
        endpwent();
        rval =  pw->pw_passwd;
@@ -37,20 +53,23 @@ get_password() { /* only run as root */
                rval = sp->sp_pwdp;
        }
 #endif
+
        /* drop privileges */
-       if(setgid(pw->pw_gid) < 0 || setuid(pw->pw_uid) < 0) {
-               fputs("slock: cannot drop privileges\n",stdout);
-               exit(EXIT_FAILURE);
-       }
+       if(setgid(pw->pw_gid) < 0 || setuid(pw->pw_uid) < 0)
+               eprint("slock: cannot drop privileges\n");
        return rval;
 }
+#endif
 
 int
 main(int argc, char **argv) {
        char curs[] = {0, 0, 0, 0, 0, 0, 0, 0};
        char buf[32], passwd[256];
        int num, screen;
+
+#ifndef HAVE_BSD_AUTH
        const char *pws;
+#endif
        unsigned int len;
        Bool running = True;
        Cursor invisible;
@@ -62,15 +81,17 @@ main(int argc, char **argv) {
        XEvent ev;
        XSetWindowAttributes wa;
 
-       if((argc > 1) && !strncmp(argv[1], "-v", 3)) {
-               fputs("slock-"VERSION", © 2006-2007 Anselm R. Garbe\n", stdout);
-               exit(EXIT_SUCCESS);
-       }
+       if((argc == 2) && !strcmp("-v", argv[1]))
+               eprint("slock-"VERSION", © 2006-2008 Anselm R Garbe\n");
+       else if(argc != 1)
+               eprint("usage: slock [-v]\n");
+
+#ifndef HAVE_BSD_AUTH
        pws = get_password();
-       if(!(dpy = XOpenDisplay(0))) {
-               fputs("slock: cannot open display\n", stderr);
-               exit(EXIT_FAILURE);
-       }
+#endif
+
+       if(!(dpy = XOpenDisplay(0)))
+               eprint("slock: cannot open display\n");
        screen = DefaultScreen(dpy);
        root = RootWindow(dpy, screen);
 
@@ -104,7 +125,9 @@ main(int argc, char **argv) {
        XSync(dpy, False);
 
        /* main event loop */
-       while(running && !XNextEvent(dpy, &ev))
+       while(running && !XNextEvent(dpy, &ev)) {
+               if(len == 0) 
+                       DPMSForceLevel(dpy, DPMSModeOff);
                if(ev.type == KeyPress) {
                        buf[0] = 0;
                        num = XLookupString(&ev.xkey, buf, sizeof buf, &ksym, 0);
@@ -115,7 +138,14 @@ main(int argc, char **argv) {
                        switch(ksym) {
                        case XK_Return:
                                passwd[len] = 0;
-                               if((running = strcmp(crypt(passwd, pws), pws)) != 0)
+
+#ifdef HAVE_BSD_AUTH
+                               running = !auth_userokay(getlogin(), NULL, "auth-xlock", passwd);
+#else
+                               running = strcmp(crypt(passwd, pws), pws);
+#endif
+
+                               if (running != 0)
                                        XBell(dpy, 100);
                                len = 0;
                                break;
@@ -134,6 +164,7 @@ main(int argc, char **argv) {
                                break;
                        }
                }
+       }
        XUngrabPointer(dpy, CurrentTime);
        XFreePixmap(dpy, pmap);
        XDestroyWindow(dpy, w);
@@ -1,4 +1,4 @@
-/* © 2006-2007 Anselm R. Garbe <garbeam at gmail dot com>
+/* © 2006-2008 Anselm R Garbe <garbeam at gmail dot com>
  * See LICENSE file for license details. */
 #define _XOPEN_SOURCE 500